#ifndef EMS_FUNC_OBJECTS
#define EMS_FUNC_OBJECTS

/*
   NAME: ems_funcobjects.cpp
   SINN:
   //
   INFO: Funktions-Objekte sind typischerweise besser "inline" benutzbar als Funktionen
   TODO:
*/


#include "ems_tupel.h"

/***********************************************************/

namespace EMS {

/////////////////////////////////////////////////////////////


/* #[ struct value_traits<> : */
template<class> struct value_traits;

template<class T> struct value_traits<T const>
{
    typedef  T  fundamental_t;
};

template<class T> struct value_traits
{
    typedef  T  fundamental_t;
};
/* #]  : */

/**/
/**/

/* #[ apply<,,>(C1 &  c1,  [Op  op,]  C2 const &  c2 ) -> void : */
// nur fr Funktions-Objekte ...
template<class Op, class C1, class C2>
inline
void
apply( C1 &  c1,  C2 const &  c2 )
{
    typename C1::      iterator  b1_i (c1.begin()), e1_i (c1.end());
    typename C2::const_iterator                     e2_ic(c2.end());

    while( b1_i != e1_i ) Op()( *--e1_i, *--e2_ic );
}

/**/

// ... AUCH fr Funktionen
template<class Op, class C1, class C2>
inline
void
apply( C1 &  c1,  Op  op,  C2 const &  c2 )
{
    typename C1::      iterator  b1_i (c1.begin()), e1_i (c1.end());
    typename C2::const_iterator                     e2_ic(c2.end());

    while( b1_i != e1_i ) op( *--e1_i, *--e2_ic );
}
/* #]  : */

/**/

struct    Assign { template<class T1, class T2> void  operator()( T1 &        t1,  T2 const &  t2 ) const {         t1   =  t2; } };
struct AddAssign { template<class T1, class T2> void  operator()( T1 &        t1,  T2 const &  t2 ) const {         t1  +=  t2; } };
struct SubAssign { template<class T1, class T2> void  operator()( T1 &        t1,  T2 const &  t2 ) const {         t1  -=  t2; } };
struct MulAssign { template<class T1, class T2> void  operator()( T1 &        t1,  T2 const &  t2 ) const {         t1  *=  t2; } };
struct DivAssign { template<class T1, class T2> void  operator()( T1 &        t1,  T2 const &  t2 ) const {         t1  /=  t2; } };

/**/
/**/

/* #[ class _CSClosure<,,> : */
template<class C, class S, class Op> class _CSClosure : private tupel<C const &, S>
{
private:

    typedef  tupel<C const &, S>  tupel_t;

    using tupel_t::first;
    using tupel_t::second;

public:

    typedef  typename C::value_t  element_t;				// const_iterator::base_iterator_t::value_t  ginge auch
    typedef  S                     scalar_t;

    /* #[ class const_iterator : */
    class const_iterator
    {
    private:

	typedef  typename C::const_iterator  base_iterator_t;


	base_iterator_t  mIter;
	scalar_t         mScalar;

    public:

	                  const_iterator( base_iterator_t  iter,  scalar_t  s )
			      :
			        mIter  (iter)
			      , mScalar(s)
	                  {}

	element_t             operator* ( void )                         const {           return  Op()( * mIter, mScalar ); }
	const_iterator &      operator++( void )                               { ++ mIter; return  * this;                   }
	const_iterator &      operator--( void )                               { -- mIter; return  * this;                   }
	bool                  operator!=( const_iterator const &  iter ) const {           return  mIter != iter.mIter;      }
    };
    /* #]  : */


                    _CSClosure( C const &  c,  S  s ) : tupel_t(c, s) {}

    const_iterator       begin( void ) const { return  const_iterator(first.begin(), second); }
    const_iterator         end( void ) const { return  const_iterator(first.  end(), second); }
};
/* #]  : */

/* #[ class _CCClosure<,,> : */
template<class C1, class C2, class Op> class _CCClosure : private tupel<C1 const &, C2 const &>
{
private:

    typedef  tupel<C1 const &, C2 const &>  tupel_t;

    using tupel_t::first;
    using tupel_t::second;

public:

    typedef  typename C1::value_t  element_t;				// const_iterator::base_iterator_t::value_t  ginge auch

    /* #[ class const_iterator : */
    class const_iterator
    {
    private:

	typedef  typename C1::const_iterator  base1_iterator_t;
	typedef  typename C2::const_iterator  base2_iterator_t;


	base1_iterator_t  mIter1;
	base2_iterator_t  mIter2;

    public:

	                  const_iterator( base1_iterator_t  iter1,  base2_iterator_t  iter2 )
			      :
			        mIter1(iter1)
			      , mIter2(iter2)
	                  {}

	element_t             operator* ( void )                         const { return  Op()( * mIter1, * mIter2 );   }
	const_iterator &      operator++( void )                               { ++ mIter1; ++ mIter2; return  * this; }
	const_iterator &      operator--( void )                               { -- mIter1; -- mIter2; return  * this; }
//	bool                  operator!=( const_iterator const &  iter ) const { return  }
    };
    /* #]  : */


                    _CCClosure( C1 const &  c1,  C2 const &  c2 ) : tupel_t(c1, c2) {}

    const_iterator       begin( void ) const { return  const_iterator(first.begin(), second.begin()); }
    const_iterator         end( void ) const { return  const_iterator(first.  end(), second.  end()); }
};
/* #]  : */

/**/

struct Add       { template<class T1, class T2> T1    operator()( T1 const &  t1,  T2 const &  t2 ) const { return  t1 + t2;    } };
struct Sub       { template<class T1, class T2> T1    operator()( T1 const &  t1,  T2 const &  t2 ) const { return  t1 - t2;    } };
struct Mul       { template<class T1, class T2> T1    operator()( T1 const &  t1,  T2 const &  t2 ) const { return  t1 * t2;    } };
struct Div       { template<class T1, class T2> T1    operator()( T1 const &  t1,  T2 const &  t2 ) const { return  t1 / t2;    } };

/**/

template<class C, class S> inline  _CSClosure<C,S,Add>  operator+( C const &  c,  S  s ) { return  _CSClosure<C,S,Add>(c, s); }
template<class C, class S> inline  _CSClosure<C,S,Sub>  operator-( C const &  c,  S  s ) { return  _CSClosure<C,S,Sub>(c, s); }
template<class C, class S> inline  _CSClosure<C,S,Mul>  operator*( C const &  c,  S  s ) { return  _CSClosure<C,S,Mul>(c, s); }
template<class C, class S> inline  _CSClosure<C,S,Div>  operator/( C const &  c,  S  s ) { return  _CSClosure<C,S,Div>(c, s); }


template<class C1, class E2, template<class> class C2> inline  _CCClosure<C1,C2<E2>,Add>
operator+( C1 const &  c1,  C2<E2> const &  c2 ) { return  _CCClosure<C1,C2<E2>,Add>(c1, c2); }

template<class C1, class E2, template<class> class C2> inline  _CCClosure<C1,C2<E2>,Sub>
operator-( C1 const &  c1,  C2<E2> const &  c2 ) { return  _CCClosure<C1,C2<E2>,Sub>(c1, c2); }

template<class C1, class E2, template<class> class C2> inline  _CCClosure<C1,C2<E2>,Mul>
operator*( C1 const &  c1,  C2<E2> const &  c2 ) { return  _CCClosure<C1,C2<E2>,Mul>(c1, c2); }

template<class C1, class E2, template<class> class C2> inline  _CCClosure<C1,C2<E2>,Div>
operator/( C1 const &  c1,  C2<E2> const &  c2 ) { return  _CCClosure<C1,C2<E2>,Div>(c1, c2); }

/**/
/**/

/* #[ operator ?= <,> : */
// "operator=" muss eine nichtstatische Elementfunktion sein

template<class C1, class C2> inline
C1 &
operator += ( C1 &  c1,  C2 const &  c2 ) { apply<AddAssign>( c1, c2 ); return  c1; }

/**/

template<class C1, class C2> inline
C1 &
operator -= ( C1 &  c1,  C2 const &  c2 ) { apply<SubAssign>( c1, c2 ); return  c1; }

/**/

template<class C1, class C2> inline
C1 &
operator *= ( C1 &  c1,  C2 const &  c2 ) { apply<MulAssign>( c1, c2 ); return  c1; }

/**/

template<class C1, class C2> inline
C1 &
operator /= ( C1 &  c1,  C2 const &  c2 ) { apply<DivAssign>( c1, c2 ); return  c1; }
/* #]  : */

/**/
/**/

/* #[ scalar_product : */
template<class C1, class C2>
inline
typename C1::value_t
scalar_product( C1 const &  c1,  C2 const &  c2 )
{
    typename C1::const_iterator  b1_i(c1.begin()), e1_i(c1.end());
    typename C2::const_iterator                    e2_i(c2.end());

    typename value_traits<typename C1::value_t>::fundamental_t  _s(0);

    while( b1_i != e1_i ) _s  +=  (*--e1_i * *--e2_i);

    return  _s;
}


// with assign
template<class C1, class C2, class C3>
inline
typename C1::value_t
scalar_product( C1 &  c1,  C2 const &  c2,  C3 const &  c3 )
{
    typename C1::      iterator  b1_i(c1.begin()), e1_i(c1.end());
    typename C2::const_iterator                    e2_i(c2.end());
    typename C3::const_iterator                    e3_i(c3.end());

    typename value_traits<typename C1::value_t>::fundamental_t  _s(0);

    while( b1_i != e1_i ) _s  +=  (*--e1_i  =  *--e2_i * *--e3_i);

    return  _s;
}
/* #]  : */

/**/
/**/

/* #[ for_each_in<,>( C &, [Op] ) -> void : */
// nur fr Funktions-Objekte ...
template<class Op, class C>
inline
void
for_each_in( C &  c )
{
    typename C::iterator  b_i(c.begin()),
	                  e_i(c.  end());

    while( b_i != e_i ) Op()( * -- e_i );
}

/**/

// ... AUCH fr Funktionen
template<class Op, class C>
inline
void
for_each_in( C &  c,  Op  op )
{
    typename C::iterator  b_i(c.begin()),
	                  e_i(c.  end());

    while( b_i != e_i ) op( * -- e_i );
}
/* #]  : */

/**/
/**/

// class Increment
// class Decrement

/* #[ class Scale<> : */
template<class S> class Scale
{
private:

    S  mS;

public:

                                 Scale( S  s ) : mS(s) {}

                      S     operator()( void )   const { return  mS; }
    template<class T> void  operator()( T &  t ) const {  t  *=  mS; }
};
/* #]  : */

/* #[ class SetTo<> : */
template<class S> class SetTo
{
private:

    S  mS;

public:

                                 SetTo( S  s ) : mS(s) {}

                      S     operator()( void )   const { return  mS; }
    template<class T> void  operator()( T &  t ) const {   t  =  mS; }
};
/* #]  : */

/* #[ class SetInc<> : */
// begin() + n = start + n * inc
template<class S> class SetInc
{
private:

    S  mSum;
    S  mInc;

public:

                                SetInc( S  i,  S  s =0 ) : mSum(s - i), mInc(i) {}

                      S     operator()( void )   const { return  mSum;         }
    template<class T> void  operator()( T &  t )       { t  =  mSum  +=  mInc; }
};
/* #]  : */

/* #[ class Clear : */
struct Clear
{
    template<class T> void  operator()( T &  t ) const { t  =  0; }
};
/* #]  : */

/* #[ class Print : */
/*
class Print
{
private:

    std::ostream &  mOS_r;

public:

                                Print( std::ostream &  os_r ) : mOS_r(os_r) {}

    template<class T>  void  operator()( T const &  t_r ) const { mOS_r << t_r << std::endl; }
};
/* #]  : */

/* #[ class Accumulate<> : */
template<class S> class Accumulate
{
private:

    S  mS;

public:

                            Accumulate( S  s =0 ) : mS(s) {}

                      S     operator()( void ) const   { return  mS;        }
    template<class T> void  operator()( T const &  t ) {         mS  +=  t; }
};
/* #]  : */

/**/
/**/

/* #[ sum( C const & ) -> C::value_t : */
template<class C>
inline
typename C::value_t
sum( C const &  c )
{
    typename C::const_iterator  b_i(c.begin()),
	                        e_i(c.  end());

    typename value_traits<typename C::value_t>::fundamental_t  _s(0);

    while( b_i != e_i ) _s  +=  *--e_i;

    return  _s;
}
/* #]  : */


/////////////////////////////////////////////////////////////

} // namespace EMS

/***********************************************************/

#endif // EMS_FUNC_OBJECTS
