function time_series(figure_no);
% function time_series(figure_no);
% figure_no = number of figure in paper (figure_no = 1,2,3)
% (See time_series_A05.m and time_series_A15.m for Figure 4 in the paper.)
%
% This function plots optimal consumption, optimal capital stock 
% and optimal saving rate for a specified number of initial values 
% Author: Klaus Reiner Schenk-Hopp (klaus@schenk-hoppe.net)
% First version: January 18, 2005.
% last update March 20, 2010.

% Produces Figures 1 - 3 in the paper 
% "Consumption Paths under Prospect Utility in an Optimal Growth Model"
% by Reto Foellmi  (University of Bern), Rina Rosenblatt-Wisch (Swiss National Bank) 
% and Klaus Reiner Schenk-Hopp (University of Leeds)

% number of time periods shown in the figure (this parameter value can be changed here)
iter_steps = 15;


switch figure_no
  case 1
     load V_alpha_09_sigma_05_loss.mat;
  case 2
     load V_alpha_09_sigma_05_habit.mat;
  case 3
     load V_alpha_00_sigma_05.mat; 
endswitch

close all

figure(3);
title('Optimal Saving Rate')
hold on

figure(2);
title('Optimal Capital Stock')
hold on

figure(1);
title('Optimal Consumption')
hold on


% initial values for capital stocks k0 and k1
% initial reference cosumption is given by c0(i) = productionF(A_prod,k0_values(i)) - k1_values(i)

k0_values = [.005 .3 .3];
k1_values = productionF(A_prod,k0_values) - [.01 .05 .45];


for i_n = 1:length(k1_values);

  k0 = k0_values(i_n);
  k1 = k1_values(i_n);
  
  policy = opt_policy(iter_steps,k0,k1, V, x, y, max_coord,alpha_value, beta_value, sigma, phi, A_prod, 0, 0);

  if (i_n == 1) 
    figure(1)
    plot((0:iter_steps-1),productionF(A_prod,x(policy(1:iter_steps,1)))-y(policy(1:iter_steps,2)),'b--','LineWidth',3);
    figure(2)
    plot((0:iter_steps-1),x(policy(1:iter_steps,1)),'b--','LineWidth',3);
    figure(3)
    plot((0:iter_steps-1),y(policy(1:iter_steps,2))./productionF(A_prod,x(policy(1:iter_steps,1))),'b--','LineWidth',3);
    axis([0 iter_steps-1 0 1]);
  end

  if ((i_n > 1) & (i_n < length(k1_values)))
    figure(1)
    plot((0:iter_steps-1),productionF(A_prod,x(policy(1:iter_steps,1)))-y(policy(1:iter_steps,2)),'k-','LineWidth',2);
    figure(2)
    plot((0:iter_steps-1),x(policy(1:iter_steps,1)),'k-','LineWidth',2);
    figure(3)
    plot((0:iter_steps-1),y(policy(1:iter_steps,2))./productionF(A_prod,x(policy(1:iter_steps,1))),'k-','LineWidth',2);
    axis([0 iter_steps-1 0 1]);
  end

  if (i_n == length(k1_values))
    figure(1)
    plot((0:iter_steps-1),productionF(A_prod,x(policy(1:iter_steps,1)))-y(policy(1:iter_steps,2)),'r-*','LineWidth',2);
    figure(2)
    plot((0:iter_steps-1),x(policy(1:iter_steps,1)),'r-*','LineWidth',2);
    figure(3)
    plot((0:iter_steps-1),y(policy(1:iter_steps,2))./productionF(A_prod,x(policy(1:iter_steps,1))),'r-*','LineWidth',2);
    axis([0 iter_steps-1 0 1]);
  end

end
