function time_series_A15(iter_steps);
%
% Produces Figures 4 (a) and  4 (b) in the paper 
% "Consumption Paths under Prospect Utility in an Optimal Growth Model" by
% by Reto Foellmi  (University of Bern), Rina Rosenblatt-Wisch (Swiss National Bank) 
% and Klaus Reiner Schenk-Hopp (University of Leeds)
%
% Author: Klaus Reiner Schenk-Hopp
% First version: January 18, 2005.
% last update August 18, 2010

close all

%%% This parameter sets the length of the time series depicted in the figure
iter_steps = 7;


fig3 = figure(3);
title('Optimal Capital Stock')
hold on

fig2 = figure(2);
title('Optimal Consumption')
hold on

fig1 = figure(1);
title('Optimal Saving Rate')
hold on


A_prod = 1.0;
k0 = 0.95^2/4.0;
c0 = sqrt(k0)-k0;
k1 = productionF(A_prod,k0) - c0;

load V_alpha_09_sigma_05_habit_A_15.mat
policy0 = opt_policy(iter_steps,k0,k1, V, x, y, max_coord,alpha_value, beta_value, sigma, phi, A_prod, c0, 1);

load V_alpha_09_sigma_05_loss_A_15.mat
policy1 = opt_policy(iter_steps,k0,k1, V, x, y, max_coord,alpha_value, beta_value, sigma, phi, A_prod, c0, 1);

load V_alpha_00_sigma_05_A_15.mat
policy2 = opt_policy(iter_steps,k0,k1, V, x, y, max_coord,alpha_value, beta_value, sigma, phi, A_prod, c0, 1);

A_prod = 1.5;


figure(3);
plot((0:iter_steps-1),x(policy1(1:iter_steps,1)),'r-*','LineWidth',2);
hold on
plot((0:iter_steps-1),x(policy2(1:iter_steps,1)),'k-','LineWidth',2);
hold on
plot((0:iter_steps-1),x(policy0(1:iter_steps,1)),'b--','LineWidth',3);
hold on

figure(2);
plot((0:iter_steps-1),[c0, productionF(A_prod,x(policy1(2:iter_steps,1)))-y(policy1(2:iter_steps,2))],'r-*','LineWidth',2);
hold on
plot((0:iter_steps-1),[c0, productionF(A_prod,x(policy2(2:iter_steps,1)))-y(policy2(2:iter_steps,2))],'k-','LineWidth',2);
hold on
plot((0:iter_steps-1),[c0, productionF(A_prod,x(policy0(2:iter_steps,1)))-y(policy0(2:iter_steps,2))],'b--','LineWidth',3);
hold on

figure(1);
plot((0:iter_steps-1),[k1/(k0+c0), y(policy1(2:iter_steps,2))./productionF(A_prod,x(policy1(2:iter_steps,1)))],'r-*','LineWidth',2);
hold on
plot((0:iter_steps-1),[k1/(k0+c0), y(policy2(2:iter_steps,2))./productionF(A_prod,x(policy2(2:iter_steps,1)))],'k-','LineWidth',2);
hold on
plot((0:iter_steps-1),[k1/(k0+c0), y(policy0(2:iter_steps,2))./productionF(A_prod,x(policy0(2:iter_steps,1)))],'b--','LineWidth',3);
hold on 


% Saves figures to eps files when using Octave
% print(fig1, 'A15_sav.eps','-depsc2')
% print(fig2, 'A15_cons.eps','-depsc2')
% print(fig3, 'A15_cap.eps','-depsc2')
